from Acquisition import aq_base
from Acquisition import aq_chain
from Acquisition import aq_inner
from Acquisition import aq_parent
from five import grok
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from zope.browser.interfaces import IBrowserView

import logging
import zExceptions

log = logging.getLogger(__name__)

grok.templatedir("templates")


class Error(grok.View):
    grok.context(Exception)
    grok.layer(NuPloneSkin)
    grok.name("index.html")
    grok.template("error_generic")

    def update(self):
        self.exception = aq_inner(self.context)
        self.context = context = aq_parent(self)
        if IBrowserView.providedBy(context):
            # NotFound errors can have extra aq wrapping
            self.context = context = aq_parent(context)
        try:
            log.exception("Error at %s", repr(context))
        except zExceptions.Unauthorized:
            pass


class NotFound(Error):
    grok.context(zExceptions.NotFound)
    grok.template("error_notfound")


class Unauthorized(Error):
    grok.context(zExceptions.Unauthorized)
    grok.template("error_unauthorized")

    def authenticate(self):
        """Try to authenticate the user manually, since ZPublisher dropped the
        user whenn it failed to validate access."""
        for parent in aq_chain(aq_inner(self.context)):
            if hasattr(aq_base(parent), "acl_users"):
                uf = parent.acl_users
                try:
                    return uf.validate(self.request, None, roles=["Anonymous"])
                except zExceptions.Unauthorized:
                    pass

    def update(self):
        super(Unauthorized, self).update()
        self.authenticate()



# coding=utf-8
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneWithPackageLayer

import plonetheme.nuplone


FIXTURE = PloneWithPackageLayer(
    zcml_filename="configure.zcml",
    zcml_package=plonetheme.nuplone,
    gs_profile_id="plonetheme.nuplone:default",
    name="plonetheme.nuplone:fixture"
)

NUPLONE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE, ), name="NuPlone:Integration"
)


NUPLONE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE, ), name="NuPlone:Functional"
)


NUPLONE_FUNCTIONAL_WITH_PAC_TESTING = FunctionalTesting(
    bases=(
        PLONE_APP_CONTENTTYPES_FIXTURE,
        FIXTURE,
    ),
    name="NuPlone:FunctionalWithPAC",
)


__all__ = ['NUPLONE_INTEGRATION_TESTING', 'NUPLONE_FUNCTIONAL_TESTING']



# -*- coding: UTF-8 -*-
""" StatusMessage adapter tests.
"""
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from plonetheme.nuplone.testing import NUPLONE_INTEGRATION_TESTING
from Products.statusmessages.interfaces import IStatusMessage
from unittest import TestCase
from webhelpers.html.builder import literal
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.interface import directlyProvides
from zope.publisher.base import RequestDataMapper
from zope.publisher.base import RequestDataProperty
from zope.publisher.browser import TestRequest as TestRequestBase


class CookieMapper(RequestDataMapper):
    _mapname = '_cookies'

    def __setitem__(self, key, value):
        pass


class TestRequest(TestRequestBase):
    """Zope 3's TestRequest doesn't support item assignment, but Zope 2's
    request does.
    """

    def __setitem__(self, key, value):
        pass

    cookies = RequestDataProperty(CookieMapper)


class TestHTMLStatusMessages(TestCase):
    layer = NUPLONE_INTEGRATION_TESTING

    def testAdapter(self):
        """ Test status messages
            First some boilerplate.
        """
        request = self.request = TestRequest()

        directlyProvides(request, NuPloneSkin)

        # Now lets make sure we can actually adapt the request.
        status = IStatusMessage(self.request)
        self.assertEquals(IStatusMessage.providedBy(status), True)
        assert (hasattr(status, 'addHTMLStatusMessage'))

        # We also need the request to be annotatable:
        directlyProvides(self.request, IAttributeAnnotatable)

        # Make sure there's no stored message.
        self.assertEquals(len(status.show()), 0)

        # Add one message
        status.add(u'test', type=u'info')

        # Now check the results
        messages = status.show()
        self.assertEquals(len(messages), 1)
        self.assertEquals(messages[0].message, u'test')
        self.assertEquals(messages[0].type, u'info')

        # Make sure messages are removed
        self.assertEquals(len(status.show()), 0)

        # Since we accessed the message prior to publishing the page, we must
        # ensure that the messages have been removed from the cookies
        self.assertEquals(len(status.show()), 0)

        # Now we repeat the test, only this time we publish the page prior to
        # retrieving the messages

        # Add one message
        status.add(u'test', type=u'info')

        # Now check the results
        messages = status.show()
        self.assertEqual(len(messages), 1)
        self.assertEquals(messages[0].message, u'test')
        self.assertEquals(messages[0].type, u'info')

        # Make sure messages are removed
        self.assertEquals(len(status.show()), 0)

        # Add two messages
        status.add(u'test', type=u'info')
        status.add(u'test1', u'warn')

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 2)
        test = messages[1]
        self.assertEquals(test.message, u'test1')
        self.assertEquals(test.type, u'warn')

        # Make sure messages are removed again
        self.assertEquals(len(status.show()), 0)

        # Add two identical messages
        status.add(u'test', type=u'info')
        status.add(u'test', type=u'info')

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.message, u'test')
        self.assertEquals(test.type, u'info')

        # Make sure messages are removed again
        self.assertEquals(len(status.show()), 0)

        # Test incredibly long messages:
        status.add(u'm' * 0x400, type=u't' * 0x20)

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)

        test = messages[0]
        assert (test.message == u'm' * 0x3FF)
        assert (test.type == u't' * 0x1F)

        # Add one HTML messages
        status.addHTML(u'test', type=u'success')

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals(test.message, u'test')
        self.assertEquals(type(test.message), literal)

        # Add two HTML messages
        status.addHTML(u'test', type=u'info')
        status.addHTML(u'test1', u'warn')

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 2)
        test = messages[1]
        self.assertEquals(test.message, u'test1')
        self.assertEquals(type(test.message), literal)
        self.assertEquals(test.type, u'warn')

        # add two identical messages
        status.addHTML(u'test', type=u'info')
        status.addHTML(u'test', type=u'info')

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.message, u'test')
        self.assertEquals(type(test.message), literal)
        self.assertEquals(test.type, u'info')

        # Make sure messages are removed again
        self.assertEquals(len(status.show()), 0)

        # Test incredibly long messages:
        status.addHTML(u'm' * 0x400, type=u't' * 0x20)

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 1)

        test = messages[0]
        self.assertEquals(type(test.message), literal)
        assert (test.message == '%s' % (u'm' * 0x3FF))
        assert (test.type == u't' * 0x1F)

        # Add two mixed messages
        status.add(u'test', type=u'info')
        status.addHTML(u'test1', u'warn')

        # And check the results again
        messages = status.show()
        self.assertEquals(len(messages), 2)
        test = messages[0]
        self.assertEquals(test.message, u'test')
        self.assertNotEquals(type(test.message), literal)
        self.assertEquals(test.type, u'info')

        test = messages[1]
        self.assertEquals(test.message, u'test1')
        self.assertEquals(type(test.message), literal)
        self.assertEquals(test.type, u'warn')

        # Add a more complicated html message
        status.addHTML(
            u'You can go <a href="http://plone.org">here</a>.',
            type=u'success'
        )
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals(
            test.message,
            u'You can go <a href="http://plone.org" rel="nofollow" target="_blank">here</a>.'  # noqa: E501
        )
        self.assertEquals(type(test.message), literal)

        # Add html message with disallowed tags
        status.addHTML(
            u'<p>You can go <a href="http://plone.org">here</a>.</p>',
            type=u'success'
        )
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals(
            test.message,
            u'You can go <a href="http://plone.org" rel="nofollow" target="_blank">here</a>.'  # noqa: E501
        )
        self.assertEquals(type(test.message), literal)

        status.addHTML(
            u"<script type=\"javascript\">alert('hello')</script>",
            type=u'success'
        )
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals(test.message, '')
        self.assertEquals(type(test.message), literal)

        status.addHTML(
            u'<a href="data:text/html;base64,PHNjcmlwdD5hbGVydCgidGVzdCIpOzwvc2NyaXB0Pg==">click me</a>',  # noqa: E501
            type=u'success'
        )
        messages = status.show()
        self.assertEquals(len(messages), 1)
        test = messages[0]
        self.assertEquals(test.type, u'success')
        self.assertEquals(test.message, '<a href="">click me</a>')
        self.assertEquals(type(test.message), literal)



from chameleon.astutil import Static
from chameleon.astutil import Symbol
from chameleon.codegen import template
from chameleon.tales import StringExpr
from plonetheme.nuplone.tiles.tile import getTile
from plonetheme.nuplone.utils import SimpleLiteral

import five.pt.engine
import logging


log = logging.getLogger(__name__)


class TileProviderTraverser(object):

    def __call__(self, context, request, name):
        tile = getTile(context, request, name)
        if tile is None:  # XXX Use custom exception?
            log.warn('Request for unknown tile %s', name)
            return u''
        return SimpleLiteral(tile())


class TileExpression(StringExpr):
    render_tile = Static(
        template("cls()", cls=Symbol(TileProviderTraverser), mode='eval')
    )

    def __call__(self, target, engine):
        assignment = super(TileExpression, self).__call__(target, engine)
        return assignment + template(
            'target = render_tile(context, request, target.strip())',
            target=target,
            render_tile=self.render_tile
        )


five.pt.engine.Program.secure_expression_types['tile'] = TileExpression
five.pt.engine.Program.expression_types['tile'] = TileExpression



from AccessControl import getSecurityManager
from Acquisition import aq_inner
from five import grok
from zExceptions import Unauthorized
from zope.interface import Interface

import Globals



class WhoAmI(grok.View):
    """Simple view to help debug authorisation problems. This view returns
    the id and login name of the current user and all his roles, both global
    and within the current context.

    This view is only accessible when Zope is running in debug mode.
    """
    grok.context(Interface)
    grok.name("whoami")
    grok.require("zope2.Public")

    def update(self):
        if not Globals.DevelopmentMode:
            raise Unauthorized

    def render(self):
        user = getSecurityManager().getUser()
        output = []
        output.append("User id: %s" % user.getId())
        output.append("Login  : %s" % user.getUserName())
        output.append("Roles  : %s" % " ".join(user.getRoles()))
        output.append(
            "Local R: %s" %
            " ".join(user.getRolesInContext(aq_inner(self.context)))
        )

        self.request.response.setHeader("Content-Type", "text/plain")
        return "\n".join(output)



from Acquisition import aq_inner
from Acquisition import aq_parent
from five import grok
from OFS.CopySupport import CopyError
from OFS.interfaces import ICopyContainer
from OFS.interfaces import ICopySource
from plonetheme.nuplone import MessageFactory as _
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from plonetheme.nuplone.utils import checkPermission
from Products.statusmessages.interfaces import IStatusMessage
from ZODB.POSException import ConflictError
from zope.component import getMultiAdapter
from zope.interface import Interface

import zExceptions

grok.templatedir("templates")


class Copy(grok.View):
    grok.context(ICopySource)
    grok.layer(NuPloneSkin)
    grok.require("zope2.CopyOrMove")
    grok.name("copy")

    def render(self):
        context = aq_inner(self.context)
        container = aq_parent(context)
        if not context.cb_isCopyable():
            raise zExceptions.Unauthorized

        container.manage_copyObjects(context.getId(), self.request)
        flash = IStatusMessage(self.request).addStatusMessage
        flash(_(u"message_copy_sucess", default=u"Copied"), "notice")
        self.request.response.redirect(context.absolute_url())


class Cut(grok.View):
    grok.context(ICopySource)
    grok.layer(NuPloneSkin)
    grok.require("zope2.CopyOrMove")
    grok.name("cut")

    def render(self):
        context = aq_inner(self.context)
        container = aq_parent(context)
        flash = IStatusMessage(self.request).addStatusMessage

        try:
            container.manage_cutObjects(context.getId(), self.request)
            flash(_("message_cut_success", default=u"Cut."), "notice")
        except CopyError:
            flash(
                _(
                    "message_cut_invalid",
                    default=u"It is not possible to move this object."
                ), "error"
            )

        self.request.response.redirect(context.absolute_url())


class Paste(grok.View):
    grok.context(ICopyContainer)
    grok.layer(NuPloneSkin)
    grok.require("zope2.CopyOrMove")
    grok.name("paste")

    def render(self):
        context = aq_inner(self.context)
        flash = IStatusMessage(self.request).addStatusMessage
        if not context.cb_dataValid():
            raise zExceptions.Unauthorized

        try:
            context.manage_pasteObjects(REQUEST=self.request)
            flash(_("message_paste_succes", default=u"Pasted"), "success")
        except ConflictError:
            raise
        except ValueError:
            flash(
                _(
                    "message_paste_valueerror",
                    default=u"You can not paste the copied data here."
                ), "error"
            )
        except zExceptions.Unauthorized:
            flash(
                _(
                    "message_paste_unauthorized",
                    default=u"You are not allowed to paste here."
                ), "error"
            )
        except CopyError as e:
            if "Insufficient Privileges" in e.message:
                raise zExceptions.Unauthorized
            flash(
                _(
                    "message_paste_generic",
                    default=u"No valid data found in the clipboard."
                ), "error"
            )

        self.request.response.redirect(context.absolute_url())


class Delete(grok.View):
    grok.context(Interface)
    grok.name("delete")
    grok.template("delete")

    def verify(self, container, context):
        if not checkPermission(container, "Delete objects"):
            raise zExceptions.Unauthorized

        return True

    def post(self):
        action = self.request.form.get("action", "cancel")
        context = aq_inner(self.context)
        flash = IStatusMessage(self.request).addStatusMessage

        if action == "cancel":
            flash(
                _("message_delete_cancel", default=u"Deletion cancelled"),
                "notice"
            )
            self.request.response.redirect(context.absolute_url())
        elif action == "delete":
            authenticator = getMultiAdapter((self.context, self.request),
                                            name=u"authenticator")
            if not authenticator.verify():
                raise zExceptions.Unauthorized

            container = aq_parent(context)
            container.manage_delObjects([context.getId()])
            flash(
                _("message_delete_success", default=u"Object removed"),
                "success"
            )
            self.request.response.redirect(container.absolute_url())

    def update(self):
        context = aq_inner(self.context)
        container = aq_parent(context)

        if not self.verify(container, context):
            return

        super(Delete, self).update()
        if self.request.method == "POST":
            self.post()



# coding=utf-8
from setuptools import find_packages
from setuptools import setup

import os


version = '1.6.0'

setup(
    name='NuPlone',
    version=version,
    description='A new user interface for Plone',
    long_description=open('README.rst').read() + '\n' +
    open(os.path.join('docs', 'changes.rst')).read(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Plone :: 5.1',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: JavaScript',
        'Programming Language :: Python :: 2 :: Only',
    ],
    keywords='',
    author='Cornelis Kolbach and Wichert Akkerman',
    author_email='wichert@wiggy.net',
    url='https://github.com/euphorie/NuPlone',
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['plonetheme'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'Plone >=4.0',
        'Zope2 >=2.12.5',
        'five.pt >= 2.2.0',
        'five.grok',
        'plone.api',
        'plone.tiles',
        'plone.z3cform',
        'plone.app.z3cform >=0.4.10dev',
        'z3c.form [extra] >=2.3.4dev',
        'plone.i18n',
        'plone.autoform',
        'plone.supermodel',
        'plone.directives.form',
        'plone.formwidget.namedfile >1.0b4',
        'z3c.appconfig',
        'p01.widget.password',
        'Products.statusmessages',
        'WebHelpers',
        'htmllaundry >=2.0',
        'zope.i18n',
        'plone.dexterity',
        'z3c.zrtresource',
    ],
    extras_require={
        'tests': [
            'plone.testing',
            'plone.app.robotframework',
            'plone.app.testing',
        ],
    },
)



# coding=utf-8
from Acquisition import aq_inner
from plone import api
from plone.tiles import Tile
from plonetheme.nuplone.utils import getNavigationRoot
from plonetheme.nuplone.utils import IS_PLONE_5
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import typesToList
from zExceptions import NotFound


class TabsTile(Tile):

    def getRoot(self):
        return getNavigationRoot(self.context)

    def update(self):
        context = aq_inner(self.context)
        contextUrl = context.absolute_url()
        navroot = self.getRoot()
        navrootPath = "/".join(navroot.getPhysicalPath())
        if IS_PLONE_5:
            use_view_types = api.portal.get_registry_record(
                'plone.types_use_view_action_in_listings',
                default=[]
            )
        else:
            portal_properties = api.portal.get_tool('portal_properties')
            site_properties = portal_properties.site_properties
            use_view_types = site_properties.getProperty(
                'typesUseViewActionInListings',
                [],
            )
        query = {}
        query["path"] = dict(query=navrootPath, depth=1)
        query["portal_type"] = typesToList(context)
        query["sort_on"] = "getObjPositionInParent"
        query["sort_order"] = "asc"
        query["is_default_page"] = False

        catalog = getToolByName(context, "portal_catalog")
        results = [{
            "id": brain.id,
            "title": brain.Title,
            "url": "%s/view" % brain.getURL()
            if brain.portal_type in use_view_types else brain.getURL(),
            "class": None
        }
                   for brain in catalog.searchResults(query)
                   if not brain.exclude_from_nav]
        current = sorted([(len(result["url"]), result["id"])
                          for result in results
                          if contextUrl.startswith(result["url"])])
        if current:
            current = current[0][1]
            for result in results:
                if result["id"] == current:
                    result["class"] = "current"
                    break

        self.tabs = results
        self.home_url = navroot.absolute_url()

    def __call__(self):
        if isinstance(self.context, NotFound):
            return ''
        self.update()
        return self.index()



from five import grok
from plone.directives import form
from plonetheme.nuplone import MessageFactory as _
from zope import schema
from zope.interface import Interface

grok.templatedir("templates")


class ExternalLinkSchema(form.Schema):
    URL = schema.URI(title=_("label_url", default=u"URL"), required=True)

    title = schema.TextLine(
        title=_("label_title", default=u"Title"), required=False
    )

    new_window = schema.Bool(
        title=_("label_new_window", default=u"Open link in new window"),
        default=True
    )


class EditLink(form.SchemaForm):
    grok.context(Interface)
    grok.require("cmf.ModifyPortalContent")
    grok.name("edit-link.html")
    grok.template("editlink")

    ignoreContext = True
    schema = ExternalLinkSchema
